package cn.superdesk.libs.cache.redis.cluster;

import java.util.HashSet;
import java.util.Set;

import cn.superdesk.libs.cache.redis.JedisProvider;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import redis.clients.jedis.BinaryJedisCluster;
import redis.clients.jedis.HostAndPort;
import redis.clients.jedis.JedisCluster;
import redis.clients.jedis.JedisPoolConfig;


/**
 * 集群 redis服务提供者
 *
 * @author superdesk
 * @description
 * @date 2015年04月23日
 */
public class JedisClusterProvider implements JedisProvider<JedisCluster, BinaryJedisCluster> {

    public static final String MODE = "cluster";

    /**
     * 出现异常最大重试次数
     * maxAttempts
     * 3次
     */
    private Integer maxRedirections = 3;
    /**
     * 获取数据超时时间
     * 3秒
     */
    private Integer soTimeout       = 3000;

    private JedisCluster       jedisCluster;
    private BinaryJedisCluster binaryJedisCluster;

    private String groupName;

    /**
     *
     */
    public JedisClusterProvider(
            String groupName,
            JedisPoolConfig jedisPoolConfig,
            String[] servers,
            int timeout,
            String password) {

        this.groupName = groupName;

        /**
         * 配置为 servers xxx.xxx.xxx.xxx:xxx,xxx.xxx.xxx.xxx:xxx
         * 传入为 xxx.xxx.xxx.xxx:xxx
         *       xxx.xxx.xxx.xxx:xxx
         */
        Set<HostAndPort> jedisClusterNodes = this.parseHostAndPort(servers);

        /* 没有填充密码版本 */
//        jedisCluster = new JedisCluster(jedisClusterNodes, timeout, maxRedirections,jedisPoolConfig);
//        binaryJedisCluster = new BinaryJedisCluster(jedisClusterNodes, timeout, maxRedirections,jedisPoolConfig);

        /**
         *  修正 redis moode cluster 密码问题 NOAUTH
         *  linjinyu
         *  20190725
         **/
        jedisCluster = new JedisCluster(
                /*Set<HostAndPort> jedisClusterNode*/jedisClusterNodes,
                /*int connectionTimeout,*/timeout,
                /*int soTimeout,*/soTimeout,
                /*int maxAttempts,*/maxRedirections,
                /*String password,*/password,
                /*final GenericObjectPoolConfig poolConfig*/jedisPoolConfig);

        binaryJedisCluster = new BinaryJedisCluster(
                /*Set<HostAndPort> jedisClusterNode,*/jedisClusterNodes,
                /*int connectionTimeout,*/timeout,
                /*int soTimeout,*/soTimeout,
                /*int maxAttempts,*/maxRedirections,
                /*String password,*/password,
                /*GenericObjectPoolConfig poolConfig*/jedisPoolConfig);
    }

    private Set<HostAndPort> parseHostAndPort(String[] servers) {
        try {
            Set<HostAndPort> haps = new HashSet<HostAndPort>();

            for (String part : servers) {
                String[]    ipAndPort = part.split(":");
                HostAndPort hap       = new HostAndPort(ipAndPort[0], Integer.parseInt(ipAndPort[1]));
                haps.add(hap);
            }

            return haps;
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Override
    public JedisCluster get() {
        return jedisCluster;
    }

    @Override
    public BinaryJedisCluster getBinary() {
        return binaryJedisCluster;
    }

    @Override
    public void release() {
    }

    @Override
    public void destroy() throws Exception {
        jedisCluster.close();
    }


    @Override
    public String mode() {
        return MODE;
    }

    @Override
    public String groupName() {
        return groupName;
    }

}