package cn.superdesk.libs.mybatis;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import cn.superdesk.libs.mybatis.plugin.cache.CacheHandler;
import cn.superdesk.libs.mybatis.plugin.pagination.PaginationHandler;
import cn.superdesk.libs.mybatis.plugin.rwseparate.RwRouteHandler;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.util.StringUtils;

import cn.superdesk.libs.common.util.ResourceUtils;

/**
 * @author superdesk
 * @description
 * @date 2018年11月22日
 */
public class MybatisConfigs {

    public static final String CRUD_DRIVER              = "superdesk.mybatis.crudDriver";
    public static final String DB_TYPE                  = "superdesk.mybatis.dbType";
    public static final String CACHE_ENABLED            = "superdesk.mybatis.cacheEnabled";
    public static final String CACHE_NULL_VALUE         = "superdesk.mybatis.nullValueCache";
    public static final String CACHE_EXPIRE_SECONDS     = "superdesk.mybatis.cacheExpireSeconds";
    public static final String CACHE_DYNAMIC_EXPIRE     = "superdesk.mybatis.dynamicExpire";
    public static final String RW_ROUTE_ENABLED         = "superdesk.mybatis.rwRouteEnabled";
    public static final String PAGINATION_ENABLED       = "superdesk.mybatis.paginationEnabled";
    public static final String INTERCEPTOR_HANDLERCLASS = "superdesk.mybatis.interceptorHandlerClass";

    private static Map<String, Properties> groupProperties = new HashMap<>();

    public static void addProperties(String group, Properties properties) {
        groupProperties.put(group, properties);
    }

    public static String getProperty(String group, String key, String defaultValue) {
        if (!groupProperties.containsKey(group)) return defaultValue;
        if (!"default".equals(group)) {
            key = group + "." + key;
        }
        return groupProperties.get(group).getProperty(key, defaultValue);
    }

    public static boolean getBoolean(String group, String key, boolean defaultValue) {
        return Boolean.parseBoolean(getProperty(group, key, String.valueOf(defaultValue)));
    }

    public static String getCrudDriver(String group) {
        return getProperty(group, CRUD_DRIVER, "default");
    }

    public static String getDbType(String group) {
        return getProperty(group, DB_TYPE, "Mysql");
    }

    public static boolean isCacheEnabled(String group) {
        return getBoolean(group, CACHE_ENABLED, false);
    }

    public static boolean isRwRouteEnabled(String group) {
        return getBoolean(group, RW_ROUTE_ENABLED, false);
    }

    public static boolean isPaginationEnabled(String group) {
        return getBoolean(group, PAGINATION_ENABLED, true);
    }

    public static String[] getHanlderNames(String group) {

        List<String> hanlders = new ArrayList<>();

        if (ResourceUtils.containsProperty(INTERCEPTOR_HANDLERCLASS)) {
            String[] customHanlderClass = StringUtils.tokenizeToStringArray(ResourceUtils.getProperty(INTERCEPTOR_HANDLERCLASS), ConfigurableApplicationContext.CONFIG_LOCATION_DELIMITERS);
            hanlders.addAll(Arrays.asList(customHanlderClass));
        }

        if (isCacheEnabled(group)) {
            hanlders.add(CacheHandler.NAME);
        }

        if (isRwRouteEnabled(group)) {
            hanlders.add(RwRouteHandler.NAME);
        }

        if (isPaginationEnabled(group)) {
            hanlders.add(PaginationHandler.NAME);
        }

        return hanlders.toArray(new String[0]);
    }
}
