package cn.superdesk.libs.mybatis.spring;

import java.lang.reflect.Method;

import cn.superdesk.libs.mybatis.datasource.DataSourceContextHolder;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.transaction.annotation.Transactional;

import cn.superdesk.libs.mybatis.plugin.cache.CacheHandler;
import cn.superdesk.libs.mybatis.plugin.rwseparate.UseMaster;

/**
 * AOP拦截器基类
 *
 * @author superdesk
 * @description
 * @date 2018年5月1日
 */
public abstract class MybatisPluginBaseSpringInterceptor {

    public abstract void pointcut();

    @Around("pointcut()")
    public Object around(ProceedingJoinPoint pjp) throws Throwable {
        try {
            MethodSignature methodSignature = (MethodSignature) pjp.getSignature();
            Method          method          = methodSignature.getMethod();

            if (method.isAnnotationPresent(UseMaster.class) || method.isAnnotationPresent(Transactional.class)) {
                DataSourceContextHolder.get().forceMaster();
            }
            return pjp.proceed();
        } catch (Exception e) {
            CacheHandler.rollbackCache();
            throw e;
        } finally {
            DataSourceContextHolder.get().clear();
        }

    }
}
