package cn.superdesk.libs.mybatis.spring;

import java.lang.reflect.Method;
import java.util.List;

import org.apache.ibatis.session.Configuration;

import cn.superdesk.libs.mybatis.MybatisConfigs;
import cn.superdesk.libs.mybatis.crud.GeneralSqlGenerator;
import cn.superdesk.libs.mybatis.parser.EntityInfo;
import cn.superdesk.libs.mybatis.parser.MybatisMapperParser;
import cn.superdesk.libs.mybatis.plugin.SuperdeskMybatisInterceptor;

/**
 * @author superdesk
 * @description
 * @date 2018年11月22日
 */
public class SuperdeskMybatisRegistry {

    public static void register(String group, Configuration configuration) throws Exception {

        if ("default".equals(MybatisConfigs.getCrudDriver(group))) {

            new GeneralSqlGenerator(group, configuration).generate();

        } else if ("mapper3".equals(MybatisConfigs.getCrudDriver(group))) {

            Class<?> helperClazz = Class.forName("tk.mybatis.mapper.mapperhelper.MapperHelper");
            Object   helper      = helperClazz.newInstance();
            Class<?> configClazz = Class.forName("tk.mybatis.mapper.entity.Config");
            Object   config      = configClazz.newInstance();
            Method   method      = configClazz.getDeclaredMethod("setNotEmpty", boolean.class);
            method.invoke(config, false);
            method = helperClazz.getDeclaredMethod("setConfig", configClazz);
            method.invoke(helper, config);

            method = helperClazz.getDeclaredMethod("registerMapper", Class.class);
            List<EntityInfo> entityInfos = MybatisMapperParser.getEntityInfos(group);
            for (EntityInfo entityInfo : entityInfos) {
                method.invoke(helper, entityInfo.getMapperClass());
            }

            method = helperClazz.getDeclaredMethod("processConfiguration", Configuration.class);
            method.invoke(helper, configuration);
        } else {

            new GeneralSqlGenerator(group, configuration).generate();

        }

        //注册拦截器
        String[] hanlderNames = MybatisConfigs.getHanlderNames(group);

        if (hanlderNames.length > 0) {
            SuperdeskMybatisInterceptor interceptor = new SuperdeskMybatisInterceptor(group, hanlderNames);
            configuration.addInterceptor(interceptor);
            interceptor.afterRegister();
        }

    }
}
