package cn.superdesk.libs.security;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.superdesk.libs.security.model.UserSession;
import cn.superdesk.libs.springweb.exception.ForbiddenAccessException;
import cn.superdesk.libs.springweb.exception.UnauthorizedException;
import cn.superdesk.libs.springweb.utils.WebUtils;
import org.springframework.web.bind.annotation.RequestMethod;

/**
 * 过滤器
 *
 * @author superdesk
 * @description
 * @date 2018年11月30日
 */
public class DelegatingFilter implements Filter {

    private static final String MSG_401_UNAUTHORIZED = "{\"code\": 401,\"msg\":\"401 Unauthorized\"}";
    private static       String MSG_403_FORBIDDEN    = "{\"code\": 403,\"msg\":\"403 Forbidden\"}";

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
    }

    @Override
    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain)
            throws IOException, ServletException {

        HttpServletRequest  request  = (HttpServletRequest) req;
        HttpServletResponse response = (HttpServletResponse) res;

        // 当跨域时 先OPTIONS 再GET POST...
        // 但到过滤这，OPTIONS会直接报没有权限，所以在此排除OPTIONS
        if(!request.getMethod().equals(RequestMethod.OPTIONS.name())){
            RequestContextHolder.set(request, response);

            UserSession session = SecurityDelegating.getCurrentSession(true);
            try {
                SecurityDelegating.doAuthorization(session, request.getRequestURI());
            } catch (UnauthorizedException e) {
                WebUtils.responseOutJson(response, MSG_401_UNAUTHORIZED);
                return;
            } catch (ForbiddenAccessException e) {
                WebUtils.responseOutJson(response, MSG_403_FORBIDDEN);
                return;
            }
        }

        chain.doFilter(req, res);
    }

    @Override
    public void destroy() {
    }

}
