package cn.superdesk.libs.security.cache;

import cn.superdesk.libs.cache.command.RedisBatchCommand;
import cn.superdesk.libs.cache.command.RedisObject;
import cn.superdesk.libs.cache.command.RedisString;
import cn.superdesk.libs.security.Cache;

/**
 * @author superdesk
 * @description
 * @date 2018年11月8日
 */
public class RedisCache implements Cache {

    private int    timeToLiveSeconds;
    private String keyPrefix;


    public RedisCache(String keyPrefix, int timeToLiveSeconds) {
        this.keyPrefix = keyPrefix + ":";
        this.timeToLiveSeconds = timeToLiveSeconds;
    }

    private String buildKey(String key) {
        return this.keyPrefix.concat(key);
    }

    @Override
    public void setString(String key, String value) {
        new RedisString(buildKey(key)).set(value, timeToLiveSeconds);
    }

    @Override
    public String getString(String key) {
        return new RedisString(buildKey(key)).get();
    }

    @Override
    public void setObject(String key, Object value) {
        new RedisObject(buildKey(key)).set(value, timeToLiveSeconds);
    }

    @Override
    public <T> T getObject(String key) {
        return new RedisObject(buildKey(key)).get();
    }

    @Override
    public void remove(String key) {
        new RedisObject(buildKey(key)).remove();
    }

    @Override
    public void removeAll() {
        RedisBatchCommand.removeByKeyPrefix(keyPrefix);
    }

    @Override
    public boolean exists(String key) {
        return new RedisObject(buildKey(key)).exists();
    }

}
